function EnKF()
close all;
global a
% Ensemble Kalman Filter (PO) applied to the sin map
% v_{j + 1} = a*sin(v_j) + \xi_j
% Code p12.m from Law, Stuart, Zygadakis, page 137
set(0,'defaultaxesfontsize',20);
format long
%% Setup
J = 1e3; % number of steps
a = 2.5; % parameter in the sim map dynamics determined by a
gamma = 1;% observational noise variance is gamma^2
gam2 = gamma^2;
sigma = 3e-1;% dynamics noise variance is sigma^2
sig2 = sigma^2;
C0 = 9e-2; % prior initial condition variance
m0 = 0; % prior initial condition mean
sd = 1; rng(sd); % choose random number seed
N = 100; % number of ensemble members
% preallocate memory for output arrayss
m = zeros(J,1);
v = m;
y = m;
c = m;
% initialize
v(1) = m0 + sqrt(C0)*randn; % initial truth
m(1) = 10*randn; % initial mean/estimate
c(1) = 10*C0; % initial covariance
H = 1; % observation operator 
% initial ensemble
U = m(1) + sqrt(c(1))*randn(1,N);
m(1) = sum(U)/N; 
%% Assimilation 
for j = 2 : J
    j1 = j - 1;
    v(j) = fun(v(j1)) + sigma*randn; % truth
    y(j) = H*v(j) + gamma*randn; %observation
    % evolve the ensemble
    Uhat = fun(U) + sigma*randn(1,N); % ensemble predict
    mhat = sum(Uhat)/N; % estimator predict
    X = Uhat - mhat;
    chat = X*X'/(N - 1); % covariance predict
    % analysis
    d = y(j) + gamma*randn(1,N) - H*Uhat; % innovation
    K = (chat*H')/(H*chat*H' + gam2); % Kalman gain
    U = Uhat + K*d; % ensemble update
    m(j) = sum(U)/N; % estimator update
    X = U - m(j);
    c(j) = X*X'/(N - 1); % covariance update
end
%% Visualization
% Plot truth, mean, standatd deviation and observations
figure; hold on; grid;
js = 21; % plot the first 21 steps
t = [0 : js - 1];
ind = [1 : js];
plot(t,v(ind),'Linewidth',2);
plot(t,m(ind),'m','Linewidth',2);
plot(t,m(ind) + sqrt(c(ind)),'r--','Linewidth',2);
plot(t,y(ind),'kx');
plot(t,m(ind) - sqrt(c(ind)),'r--','Linewidth',2);
legend('v','m','m \pm c^{1/2}','y');
xlabel('iteration, j');
title('EnKF, sin map');
% Plot covariance
figure; hold on; grid;
t = [0 : J - 1];
plot(t,c,'Linewidth',2);
plot(t,cumsum(c)./[1:J]','m','Linewidth',2);
xlabel('iteration, j');
title('EnKF Covariance, sin map');
legend('covariance','running mean covariance');
% Plot error
figure; hold on; grid;
e = v-m; % error
rme = cumsum(e.^2)./[1:J]'; % running mean error
plot(t,e.^2,'Linewidth',2);
plot(t,rme,'m','Linewidth',2);
axis([0,J,0,20]);
xlabel('iteration, j');
title('EnKF Error, sin map')
legend('squared error','running mean squared error');
% save('EnKFerror.mat','e','rme');
end

%%
function f = fun(x)
global a
f = a*sin(x);
end
