function ParticleFilter()
global a
close all;
% Particle Filter (SIRS) applied to the sin map
% v_{j + 1} = a*sin(v_j) + \xi_j
% Code p14.m from Law, Stuart, Zygadakis, page 131
set(0,'defaultaxesfontsize',20);
format long
%% Setup
J = 1e5; % number of steps
a = 2.5; % parameter in the sim map dynamics determined by a
gamma = 1;% observational noise variance is gamma^2
gam2 = gamma^2;
sigma = 3e-1;% dynamics noise variance is sigma^2
sig2 = sigma^2;
C0 = 9e-2; % prior initial condition variance
m0 = 0; % prior initial condition mean
sd = 1; rng(sd); % choose random number seed
N = 100; % number of ensemble members
% preallocate memory for output arrayss
m = zeros(J,1);
v = m;
y = m;
c = m;
% initialize
v(1) = m0 + sqrt(C0)*randn; % initial truth
m(1) = 10*randn; % initial mean/estimate
c(1) = 10*C0; % initial covariance
H = 1; % observation operator 
% initial ensemble
U = m(1) + sqrt(c(1))*randn(1,N);
m(1) = sum(U)/N; 
%% Assimilation 
for j = 2 : J
    j1 = j - 1;
    v(j) = fun(v(j1)) + sigma*randn; % truth
    y(j) = H*v(j) + gamma*randn; %observation
    % evolve the ensemble
    Uhat = fun(U) + sigma*randn(1,N); % ensemble predict
    d = y(j) - H*Uhat; % ensemble innovation
    what = exp(-0.5*(1/gam2*d.^2)); % weight update
    w = what/sum(what); % normalize predict weights
    % analysis
    % 1. resample
    ws = cumsum(w); % compute CDF of weights
    for n = 1 : N
%         ix = [];
%         while isempty(ix)
            ix = find(ws > rand,1,'first'); % resample: draw rand ~ U[0,1]
%         end
        % and find the index of the particle corresponding to the 
        % first time the CDF of the weights exceeds rand
        U(n) = Uhat(ix); % reset the n-th particle to the one with index ix
    end
    % 2. update mean and covariance
    m(j) = sum(U)/N; % estimator update
    X = U - m(j);
    c(j) = X*X'/N; % covariance update
end
%% Visualization
% Plot truth, mean, standatd deviation and observations
figure; hold on; grid;
js = 21; % plot the first 21 steps
t = [0 : js - 1];
ind = [1 : js];
plot(t,v(ind),'Linewidth',2);
plot(t,m(ind),'m','Linewidth',2);
plot(t,m(ind) + sqrt(c(ind)),'r--','Linewidth',2);
plot(t,y(ind),'kx');
plot(t,m(ind) - sqrt(c(ind)),'r--','Linewidth',2);
legend('v','m','m \pm c^{1/2}','y');
xlabel('iteration, j');
title('Particle Filter sin map');
% Plot covariance
figure; hold on; grid;
t = [0 : J - 1];
plot(t,c,'Linewidth',2);
plot(t,cumsum(c)./[1:J]','m','Linewidth',2);
xlabel('iteration, j');
title('Particle Filter Covariance, sin map');
legend('covariance','running mean covariance');
% Plot error
figure; hold on; grid;
e = v-m; % error
e2 = e.^2;
rme = cumsum(e2)./[1:J]'; % running mean error
plot(t,e2,'Linewidth',2);
plot(t,rme,'m','Linewidth',2);
axis([0,J,0,20]);
xlabel('iteration, j');
title('Particle Filter, sin map')
legend('squared error','running mean squared error');
save('PFerror.mat','e','rme');
end

%%
function f = fun(x)
global a
f = a*sin(x);
end